/*
 * UART.c
 * Based on the original PGM 
 * Created: 24.04.2013 23:03:35
 *  Author: jens
 *
 * Modified and adapted to fit into programm by
 * Hartmut Burger 
 * 01.10.2014
 */

#include "init.h"
#include "ProtocollData.h"
#define F_CPU 8000000UL  // 8 MHz

/*Very Important - change F_CPU to match target clock
  Note: default AVR CLKSEL is 1MHz internal RC
  This program transmits ??continously?? on USART. Interrupt is used for
  Receive charactor, which is then transmitted instead. LEDs are used
  as a test. RX routine is included but not used.
  Change USART_BAUDRATE constant to change Baud Rate
*/

#include <avr/io.h>

#include <avr/interrupt.h>

// Define baud rate
#define F_CPU 8000000UL
#define USART_BAUDRATE 19200
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)




volatile unsigned char value;
/* This variable is volatile so both main and RX interrupt can use it.
It could also be a uint8_t type */

/* Interrupt Service Routine for Receive Complete
NOTE: vector name changes with different AVRs see
Help - AVR-Libc reference - Library Reference - <avr/interrupt.h>: 
Interrupts
for vector names other than USART_RXC_vect for ATmega32 */

/*/ISR(USART0_RX_vect){

   value = UDR0;						 //read UART register into value
   PORTC= ~value;						 // output inverted value on LEDs (0=on)
}*/

extern void USART_Init(void){
   // Set baud rate
   UBRR0L = BAUD_PRESCALE;				// Load lower 8-bits into the low byte of the UBRR0L register
   UBRR0H = (BAUD_PRESCALE >> 8);
										/* Load upper 8-bits into the high byte of the UBRR register
										   Default frame format is 8 data bits, no parity, 1 stop bit
										   to change use UCSRC0B*/

										// Enable receiver and transmitter and receive complete interrupt
  UCSR0B = ((1<<TXEN0)|(1<<RXEN0) | (1<<RXCIE0));
}


extern void USART_SendByte(uint8_t u8Data){

  
  while((UCSR0A &(1<<UDRE0)) == 0);			// Wait until last byte has been transmitted

  // Transmit data
  UDR0 = u8Data;
}


							
							
extern uint8_t USART_ReceiveByte(){
  while((UCSR0A &(1<<RXC0)) == 0);			// Wait until a byte has been received and return received data
  return UDR0;
}


				
int Startsequence1 = 0;						// Startsequence 1 first byte Test //
int Startsequence2 = 0;						// Startsequence 2 second byte Test //		

void StartCommunication(int x ){
   USART_Init();							// Initialise USART
   cli();									// disable all interrupts
   
  
  
   while(StartComm == 0 )					// Repeat loop until Transmission completed //
   {    
	  USART_ReceiveByte();					// get the propper Sign from the Frontboard /
	  	 
	  if (UDR0 == prot_BEGIN && Startsequence1 == 0 && Startsequence2 == 0 )	// if "protbegin" received from Frontboard //
				{				
				USART_SendByte(prot_ACK);														// send "ProtACK" as Answer to Fronboard //
				Startsequence1= 1;																// Startsequence1 DONE //
				}														
				else
				{
				Startsequence1 = 0;																// End if Startsequence 1 //
				
				}
	 		
	 																					
	  if (UDR0 == prot_Receive_Data && Startsequence1 == 1 && Startsequence2 == 0 );
				{																		// if " Data Received" received from Frontboard //
				USART_SendByte(prot_ACK);												// send "ProtACK" as Answer to Fronboard //
				asm volatile ("nop");													// wait a tick for Answer from Frontboard //
				USART_ReceiveByte();													// ask for new DATA //
				if (UDR0 > 0 ){
					prot_code_FAULT_receicve= UDR0;
					USART_SendByte(prot_ACK);										// if any Faultcode arose //
					FaultHandler(prot_code_FAULT_receicve,0);
					asm volatile ("nop");
					}																	// call Faulthandler //
					else if (prot_code_FAULT_receicve == 0)
					{
					Startsequence2 = 1;													// Startsequence2 DONE //
					asm volatile ("nop");	
					}
					else
					{
					Startsequence2 = 1;													// Startsequence2 DONE //
														
					}																	// End if Startsequence 2 //
					}
	  if (Startsequence1 == 1 && Startsequence2 == 1);
			{
		  StartComm = 1;
		  Startsequence1 = 0;
		  Startsequence2 = 0;
		  asm volatile ("nop");
			}	  
	StartComm = 1;
	sei();																				// enable all interrupts
	}																					// End While //
}
																						
																						// End StartCom //